import { createLogger } from "vite";
import CleanCSS from "clean-css";
function InjectCssToJsPlugin({ beforeInjectCss } = {}) {
  const logger = createLogger();
  let buildConfig;
  const cssSourceMap = /* @__PURE__ */ new Map();
  const cssTags = /* @__PURE__ */ new Set();
  let isSkip = false;
  const compressCss = (chunk, cssCode) => {
    let processedCssCode = cssCode;
    let moduleIds = chunk.moduleIds;
    if (!moduleIds) {
      moduleIds = Object.keys(chunk.modules);
    }
    const isFromNodeModules = moduleIds.filter((id) => id.includes("node_modules")).length;
    if (!isFromNodeModules) {
      try {
        const output = new CleanCSS({
          level: 2,
          compatibility: "ie7"
        }).minify(cssCode);
        processedCssCode = output.styles;
      } catch (error) {
        logger.error("Failed to compress CSS: ", { error });
      }
    }
    return processedCssCode;
  };
  const checkIsSkip = () => {
    if ((buildConfig == null ? void 0 : buildConfig.cssCodeSplit) === false) {
      logger.warn(`The 'cssCodeSplit' option is set to true in the 'build' configuration of userConfig, which is not supported by this plugin, so it is skipped.`);
      isSkip = true;
    }
  };
  return {
    name: "vite-plugin-inject-css-to-js",
    apply: "build",
    enforce: "post",
    config(userConfig) {
      buildConfig = userConfig.build;
      checkIsSkip();
    },
    transformIndexHtml: {
      enforce: "post",
      transform: function(html, ctx) {
        var _a;
        if (isSkip) {
          return html;
        }
        const analyzedChunk = /* @__PURE__ */ new Map();
        const getCssTagsForChunk = (chunk, seen = /* @__PURE__ */ new Set()) => {
          const tags = [];
          if (!analyzedChunk.has(chunk)) {
            analyzedChunk.set(chunk, 1);
            chunk.imports.forEach((file) => {
              var _a2;
              const importee = (_a2 = ctx.bundle) == null ? void 0 : _a2[file];
              if ((importee == null ? void 0 : importee.type) === "chunk") {
                tags.push(...getCssTagsForChunk(importee, seen));
              }
            });
          }
          chunk.viteMetadata.importedCss.forEach((file) => {
            if (!seen.has(file)) {
              seen.add(file);
              tags.push({
                filename: file
              });
            }
          });
          return tags;
        };
        if (((_a = ctx.chunk) == null ? void 0 : _a.type) === "chunk" && ctx.chunk.isEntry) {
          getCssTagsForChunk(ctx.chunk).forEach((cssTag) => {
            cssTags.add(cssTag.filename);
          });
        }
        return html;
      }
    },
    async generateBundle(_, bundle) {
      var _a;
      if (isSkip) {
        return;
      }
      const emittedFileList = [];
      let bundleKeys = Object.keys(bundle);
      for (let i = 0; i < bundleKeys.length; i++) {
        const key = bundleKeys[i];
        const chunk = bundle[key];
        if (chunk.type === "asset" && chunk.fileName.endsWith(".css")) {
          cssSourceMap.set(chunk.fileName, chunk.source);
          delete bundle[key];
        }
      }
      bundleKeys = Object.keys(bundle);
      for (let i = 0; i < bundleKeys.length; i++) {
        const key = bundleKeys[i];
        const chunk = bundle[key];
        if (chunk.type === "chunk" && ((_a = chunk == null ? void 0 : chunk.viteMetadata) == null ? void 0 : _a.importedCss.size)) {
          const importedCss = Array.from(chunk.viteMetadata.importedCss);
          for (let cssId of importedCss) {
            let cssCode = cssSourceMap.get(cssId);
            if (!cssCode) {
              continue;
            }
            cssCode = compressCss(chunk, cssCode);
            if (cssTags.has(cssId)) {
              if (!emittedFileList.includes(cssId)) {
                emittedFileList.push(cssId);
                this.emitFile({ type: "asset", fileName: cssId, source: cssCode });
              }
            } else {
              const initialCode = chunk.code;
              const originalCssCode = cssCode;
              if (cssCode && typeof beforeInjectCss === "function") {
                cssCode = await beforeInjectCss(cssCode);
              }
              if (!cssCode || typeof cssCode !== "string") {
                logger.warn(`The beforeInjectCss hook returns content that is not a string or is empty, reverting to the original content.`);
                cssCode = JSON.stringify(originalCssCode.trim());
              }
              chunk.code = `(function(){ try {var elementStyle = document.createElement('style'); elementStyle.appendChild(document.createTextNode(${cssCode}));document.head.appendChild(elementStyle);} catch(e) {console.error('style-injected-by-js', e);} })(); ${initialCode}`;
            }
          }
          chunk.viteMetadata.importedCss.clear();
        }
      }
      cssSourceMap.clear();
    }
  };
}
export {
  InjectCssToJsPlugin
};
